from ucollections import OrderedDict
import filedb as uorm


db = uorm.DB("notes-db")

class Note(uorm.Model):

    __db__ = db
    __table__ = "note"
    __schema__ = OrderedDict([
        ("timestamp", ("TIMESTAMP", uorm.now)),
        ("archived", ("INT", 0)),
        ("content", ("TEXT", "")),
    ])

    @classmethod
    def mapkeys(cls, obj):
        return [obj.get(k) for k in cls.__schema__.keys()]

    @classmethod
    def public(cls):
        res = [x for x in cls.scan() if x.archived == 0]
        res.sort(key=lambda x: x.timestamp, reverse=True)
        return res



from . import config


if config.DB_BACKEND == "btree":
    from .models_btree import *
elif config.DB_BACKEND == "filedb":
    from .models_filedb import *
elif config.DB_BACKEND == "sqlite":
    from .models_sqlite import *
else:
    raise ValueError("Unknown DB backend")



# Autogenerated file
def render(note):
    yield """<li class=\"note col-xs-12 col-sm-6 col-lg-4\">
  <div class=\"panel panel-primary\">
    <div class=\"panel-heading\">
      """
    yield str(note[0])
    yield """
      <a
        class=\"btn btn-danger btn-xs archive-note pull-right\"
        data-note=\""""
    yield str(note[0])
    yield """\"
        href=\"archive/"""
    yield str(note[0])
    yield """\">&times;</a>
    </div>
    <div class=\"panel-body\">
      """
    yield str(note[2])
    yield """
    </div>
  </div>
</li>
"""



import picoweb
from . import models


class DBApp(picoweb.WebApp):

    def init(self):
        models.db.connect()
        models.Note.create_table(True)
        super().init()

app = DBApp(__name__)



R = {
'static/js/notes.js': b"Notes = window.Notes || {};\n\n(function(exports, $) {\n  function Editor() {\n    this.form = $('form#note-form');\n    this.editor = $('textarea#content');\n    this.container = $('ul.notes');\n    this.initialize();\n  }\n\n  Editor.prototype.initialize = function() {\n    this.setupMasonry();\n    this.setupNotes();\n    this.setupForm();\n    this.editor.focus();\n  }\n\n  Editor.prototype.setupMasonry = function() {\n    var self = this;\n    imagesLoaded(this.container, function() {\n      self.container.masonry({'itemSelector': '.note'});\n    });\n  }\n\n  Editor.prototype.setupNotes = function() {\n    var self = this;\n    $('a.archive-note').on('click', this.archiveNote);\n  }\n\n  Editor.prototype.archiveNote = function(e) {\n    e.preventDefault();\n    var elem = $(this);\n    var panel = elem.parents('.panel');\n    var self = this;\n    $.post(elem.attr('href'), function(data) {\n      if (data.success) {\n        panel.remove();\n        $('ul.notes').masonry();\n      }\n    });\n  }\n\n  Editor.prototype.setupForm = function() {\n    var self = this;\n    this.editor.on('keydown', function(e) {\n      if (e.ctrlKey && e.keyCode == 13) {\n        self.form.submit();\n      }\n    });\n    this.form.submit(function(e) {\n      e.preventDefault();\n      self.addNote();\n    });\n    this.addMarkdownHelpers();\n  }\n\n  Editor.prototype.addNote = function() {\n    var self = this;\n    this.editor.css('color', '#464545');\n    $.post(this.form.attr('target'), this.form.serialize(), function(data) {\n      if (data.success) {\n        self.editor.val('').focus();\n        listElem = $(data.note);\n        listElem.find('a.archive-note').on('click', self.archiveNote);\n        self.container.prepend(listElem);\n        self.container.masonry('prepended', listElem);\n      } else {\n        self.editor.css('color', '#dd1111');\n      }\n    });\n  }\n\n  Editor.prototype.addMarkdownHelpers = function() {\n    var self = this;\n    this.addHelper('indent-left', function(line) {\n      return '    ' + line;\n    });\n    this.addHelper('indent-right', function(line) {\n      return line.substring(4);\n    });\n    this.addHelper('list', function(line) {\n      return '* ' + line;\n    });\n    this.addHelper('bold', function(line) {\n      return '**' + line + '**';\n    });\n    this.addHelper('italic', function(line) {\n      return '*' + line + '*';\n    });\n    this.addHelper('font', null, function() {\n      self.focus().select();\n    });\n  }\n\n  Editor.prototype.addHelper = function(iconClass, lineHandler, callBack) {\n    var link = $('<a>', {'class': 'btn btn-xs'}),\n        icon = $('<span>', {'class': 'glyphicon glyphicon-' + iconClass}),\n        self = this;\n\n    if (!callBack) {\n      callBack = function() {\n        self.modifySelection(lineHandler);\n      }\n    }\n\n    link.on('click', function(e) {\n      e.preventDefault();\n      callBack();\n    });\n    link.append(icon);\n    this.editor.before(link);\n  }\n\n  Editor.prototype.modifySelection = function(lineHandler) {\n    var selection = this.getSelectedText();\n    if (!selection) return;\n\n    var lines = selection.split('\\n'),\n        result = [];\n    for (var i = 0; i < lines.length; i++) {\n      result.push(lineHandler(lines[i]));\n    }\n\n    this.editor.val(\n      this.editor.val().split(selection).join(result.join('\\n'))\n    );\n  }\n\n  Editor.prototype.getSelectedText = function() {\n    var textAreaDOM = this.editor[0];\n    return this.editor.val().substring(\n      textAreaDOM.selectionStart,\n      textAreaDOM.selectionEnd);\n  }\n\n  exports.Editor = Editor;\n})(Notes, jQuery);",
}



import uorm


db = uorm.DB("sqlite.db")


class Note(uorm.Model):

    __db__ = db
    __table__ = "note"
    __pkey__ = "timestamp"
    __schema__ = """
        CREATE TABLE note(
        timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP PRIMARY KEY,
        archived INT NOT NULL DEFAULT 0,
        content TEXT NOT NULL
        )
    """

    @classmethod
    def public(cls):
        return cls.execute("""
            SELECT * FROM note
            WHERE archived=0
            ORDER BY timestamp
        """)



from .app import app
from . import views
import gc

def main(**params):
    gc.collect()
    import ulogging
    ulogging.basicConfig(level=logging.INFO)

    # Preload templates to avoid memory fragmentation issues
    gc.collect()
    app._load_template('homepage.html')
    app._load_template('note.html')
    gc.collect()

    import micropython
    micropython.mem_info()
    app.run(debug=True, **params)

if __name__ == '__main__':
    main()



from ucollections import OrderedDict
import ujson
import utime

import btreedb as uorm
import btree


db = uorm.DB("notes.db")

class Note(uorm.Model):

    __db__ = db
    __table__ = "note"
    __schema__ = OrderedDict([
        ("timestamp", ("TIMESTAMP", uorm.now)),
        ("archived", ("INT", 0)),
        ("content", ("TEXT", "")),
    ])


    @classmethod
    def public(cls):
        print("public")
        for v in cls.__db__.db.values(None, None, btree.DESC):
            res = ujson.loads(v)
            row = cls.Row(*res)
            if row.archived:
                continue
            yield row



type_gen = type((lambda: (yield))())


def conv(it):
    yield '"'
    #yield from it
    for i in it:
        yield i.replace('"', "'").replace("\n", "\\n")
    yield '"'


def idumps(obj):
    if isinstance(obj, str):
        yield '"'
        yield obj
        yield '"'
    elif isinstance(obj, int):
        yield str(obj)
    elif isinstance(obj, dict):
        comma = False
        yield '{'
        for k, v in obj.items():
            if comma:
                yield ',\n'
            yield from idumps(k)
            yield ': '
            yield from idumps(v)
            comma = True
        yield '}'
    elif isinstance(obj, type_gen):
        yield from conv(obj)
    else:
        assert 0, repr(obj)


if __name__ == "__main__":
    def templ():
        yield from ["1", "2", "3"]
    print(list(idumps({"foo": templ()})))



#from flask import abort, jsonify, render_template, request

from .app import app
from .models import Note

import ure as re
import picoweb
from . import ijson

def get_page():
    page = request.args.get('page')
    if not page or not page.isdigit():
        return 1
    return min(int(page), 1)

@app.route('/', methods=['GET', 'POST'])
def homepage(request, response):
    if request.method == 'POST':
        print(request.headers)
        yield from request.read_form_data()
        if request.form.get('content'):
            note_id = Note.create(content=request.form['content'][0])
            note = list(Note.get_id(note_id))[0]
            print("note after create:", note)
            tmpl = app._load_template('note.html')
            yield from picoweb.start_response(response, "application/json")
            yield from response.awriteiter(ijson.idumps({'note': tmpl(note), 'success': 1}))
            return

        yield from picoweb.jsonify(response, {'success': 0})
        return

    yield from picoweb.start_response(response)
#    notes = Note.public().paginate(get_page(), 50)
    notes = Note.public()
    yield from app.render_template(response, 'homepage.html', (notes,))

@app.route(re.compile('^/archive/(.+)'), methods=['POST'])
def archive_note(request, response):
    pkey = picoweb.utils.unquote_plus(request.url_match.group(1))
    print("archive_note", pkey)
    Note.update({"timestamp": pkey}, archived=1)
    yield from picoweb.jsonify(response, {'success': True})



# Autogenerated file
def render(notes):
    yield """<!DOCTYPE html>
<html lang=\"en\">
  <head>
    <meta charset=\"utf-8\">
    <meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">
    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">
    <title>Notes</title>
    <link href=\"//netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css\" rel=\"stylesheet\">
    <link href=\"css/site.css\" rel=\"stylesheet\">
    <script src=\"//code.jquery.com/jquery-1.11.0.min.js\"></script>
    <script src=\"//netdna.bootstrapcdn.com/bootstrap/3.1.1/js/bootstrap.min.js\"></script>
    <script src=\"//imagesloaded.desandro.com/imagesloaded.pkgd.min.js\"></script>
    <script src=\"//masonry.desandro.com/masonry.pkgd.min.js\"></script>
    <script src=\"static/js/notes.js\"></script>
    <script type=\"text/javascript\">
      $(function() """
    yield """{
        new Notes.Editor();
      });
    </script>
  </head>
  <body>
    <div class=\"container content\">
      <div class=\"page-header\">
        <h1>Notes</h1>
      </div>
      <form action=\".\" class=\"form\" id=\"note-form\" method=\"post\">
        <button class=\"btn btn-primary btn-xs\" type=\"submit\">
          <span class=\"glyphicon glyphicon-plus\"></span> Add Note
        </button>
        <textarea class=\"form-control\" id=\"content\" name=\"content\"></textarea>
      </form>
      <br />
      <ul class=\"list-unstyled notes\">
        """
    for note in notes:
        yield """          """
# Autogenerated file
        def render1(note):
            yield """<li class=\"note col-xs-12 col-sm-6 col-lg-4\">
  <div class=\"panel panel-primary\">
    <div class=\"panel-heading\">
      """
            yield str(note[0])
            yield """
      <a
        class=\"btn btn-danger btn-xs archive-note pull-right\"
        data-note=\""""
            yield str(note[0])
            yield """\"
        href=\"archive/"""
            yield str(note[0])
            yield """\">&times;</a>
    </div>
    <div class=\"panel-body\">
      """
            yield str(note[2])
            yield """
    </div>
  </div>
</li>
"""
        yield from render1(note)
        yield """        """
    yield """      </ul>
      <div style=\"clear:both;\"></div>
    </div>
  </body>
</html>"""
